@@ -2,7 +2,8 @@
 {-# LANGUAGE TemplateHaskell #-}
 module Main where
 
-import Control.Lens (makeLenses, (^.), (&), (.~), (%~))
+import Lens.Micro ((^.), (&), (.~), (%~))
+import Lens.Micro.TH (makeLenses)
 import Control.Monad (void, forever)
 import Control.Concurrent (newChan, writeChan, threadDelay, forkIO)
 import Data.Default
@@ -36,14 +37,14 @@ makeLenses ''St
 data CustomEvent = VtyEvent V.Event
                  | Counter
 
-drawUI :: St -> [Widget]
+drawUI :: St -> [Widget ()]
 drawUI st = [a]
     where
         a = (str $ "Last Vty event: " <> (show $ st^.stLastVtyEvent))
             <=>
             (str $ "Counter value is: " <> (show $ st^.stCounter))
 
-appEvent :: St -> CustomEvent -> EventM (Next St)
+appEvent :: St -> CustomEvent -> EventM () (Next St)
 appEvent st e =
     case e of
         VtyEvent (V.EvKey V.KEsc []) -> halt st
@@ -56,7 +57,7 @@ initialState =
        , _stCounter = 0
        }
 
-theApp :: App St CustomEvent
+theApp :: App St CustomEvent ()
 theApp =
     App { appDraw = drawUI
         , appChooseCursor = showFirstCursor
