@@ -2,7 +2,8 @@
 {-# LANGUAGE TemplateHaskell #-}
 module Main where
 
-import Control.Lens (makeLenses, (.~), (^.), (&))
+import Lens.Micro ((.~), (^.), (&))
+import Lens.Micro.TH (makeLenses)
 import Control.Monad (void)
 import Data.Monoid
 import Data.Default
@@ -28,14 +29,14 @@ data St =
 
 makeLenses ''St
 
-drawUI :: St -> [Widget]
+drawUI :: St -> [Widget ()]
 drawUI st = [ui]
     where
         ui = vBox [ str $ "External input: \"" <> st^.stExternalInput <> "\""
                   , str "(Press Esc to quit or Space to ask for input)"
                   ]
 
-appEvent :: St -> V.Event -> EventM (Next St)
+appEvent :: St -> V.Event -> EventM () (Next St)
 appEvent st e =
     case e of
         V.EvKey V.KEsc [] -> halt st
@@ -50,7 +51,7 @@ initialState =
     St { _stExternalInput = ""
        }
 
-theApp :: App St V.Event
+theApp :: App St V.Event ()
 theApp =
     App { appDraw = drawUI
         , appChooseCursor = neverShowCursor
