@@ -1,3 +1,4 @@
+{-# LANGUAGE DeriveFunctor #-}
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 module Brick.Types.Internal
@@ -5,7 +6,6 @@ module Brick.Types.Internal
   , VisibilityRequest(..)
   , vrPositionL
   , vrSizeL
-  , Name(..)
   , Location(..)
   , locL
   , origin
@@ -18,40 +18,41 @@ module Brick.Types.Internal
   , cursorLocationL
   , cursorLocationNameL
   , Context(..)
-  , EventState
+  , EventState(..)
+  , EventRO(..)
   , Next(..)
+  , Result(..)
+  , CacheInvalidateRequest(..)
 
-  , scrollRequestsL
+  , rsScrollRequestsL
   , viewportMapL
+  , renderCacheL
+  , observedNamesL
   , vpSize
   , vpLeft
   , vpTop
+  , imageL
+  , cursorsL
+  , visibilityRequestsL
   )
 where
 
-import Control.Lens (Field1, Field2, _1, _2, Lens', makeLenses)
-import Data.String
+#if !MIN_VERSION_base(4,8,0)
 import Data.Monoid
+#endif
+
+import Lens.Micro (_1, _2, Lens')
+import Lens.Micro.TH (makeLenses)
+import Lens.Micro.Internal (Field1, Field2)
+import qualified Data.Set as S
 import qualified Data.Map as M
-import Graphics.Vty (DisplayRegion)
+import Graphics.Vty (Vty, DisplayRegion, Image, emptyImage)
+import Data.Default (Default(..))
 
 import Brick.Types.TH
 import Brick.AttrMap (AttrName, AttrMap)
 import Brick.Widgets.Border.Style (BorderStyle)
 
--- | Names of things. Used to name cursor locations, widgets, and
--- viewports.
-newtype Name = Name String
-             deriving (Eq, Show, Ord)
-
-instance IsString Name where
-    fromString = Name
-
-data RenderState =
-    RS { viewportMap :: M.Map Name Viewport
-       , scrollRequests :: [(Name, ScrollRequest)]
-       }
-
 data ScrollRequest = HScrollBy Int
                    | HScrollPage Direction
                    | HScrollToBeginning
@@ -60,12 +61,14 @@ data ScrollRequest = HScrollBy Int
                    | VScrollPage Direction
                    | VScrollToBeginning
                    | VScrollToEnd
+                   | SetTop Int
+                   | SetLeft Int
 
 data VisibilityRequest =
     VR { vrPosition :: Location
        , vrSize :: DisplayRegion
        }
-       deriving Show
+       deriving (Show, Eq)
 
 -- | Describes the state of a viewport as it appears as its most recent
 -- rendering.
@@ -87,26 +90,37 @@ data ViewportType = Vertical
                   -- ^ Viewports of this type are scrollable only horizontally.
                   | Both
                   -- ^ Viewports of this type are scrollable vertically and horizontally.
-                  deriving Show
+                  deriving (Show, Eq)
 
-type EventState = [(Name, ScrollRequest)]
+data CacheInvalidateRequest n = InvalidateSingle n
+                              | InvalidateEntire
+
+data EventState n = ES { esScrollRequests :: [(n, ScrollRequest)]
+                       , cacheInvalidateRequests :: [CacheInvalidateRequest n]
+                       }
+
+data EventRO n = EventRO { eventViewportMap :: M.Map n Viewport
+                         , eventVtyHandle :: Maybe Vty
+                         }
 
 -- | The type of actions to take upon completion of an event handler.
 data Next a = Continue a
             | SuspendAndResume (IO a)
             | Halt a
+            deriving Functor
 
 -- | Scrolling direction.
 data Direction = Up
                -- ^ Up/left
                | Down
                -- ^ Down/right
+               deriving (Show, Eq)
 
 -- | A terminal screen location.
 data Location = Location { loc :: (Int, Int)
                          -- ^ (Column, Row)
                          }
-                deriving Show
+                deriving (Show, Eq)
 
 suffixLenses ''Location
 
@@ -140,17 +154,44 @@ instance Monoid Location where
     mappend (Location (w1, h1)) (Location (w2, h2)) = Location (w1+w2, h1+h2)
 
 -- | A cursor location.  These are returned by the rendering process.
-data CursorLocation =
+data CursorLocation n =
     CursorLocation { cursorLocation :: !Location
                    -- ^ The location
-                   , cursorLocationName :: !(Maybe Name)
+                   , cursorLocationName :: !(Maybe n)
                    -- ^ The name of the widget associated with the location
                    }
                    deriving Show
 
+-- | The type of result returned by a widget's rendering function. The
+-- result provides the image, cursor positions, and visibility requests
+-- that resulted from the rendering process.
+data Result n =
+    Result { image :: Image
+           -- ^ The final rendered image for a widget
+           , cursors :: [CursorLocation n]
+           -- ^ The list of reported cursor positions for the
+           -- application to choose from
+           , visibilityRequests :: [VisibilityRequest]
+           -- ^ The list of visibility requests made by widgets rendered
+           -- while rendering this one (used by viewports)
+           }
+           deriving Show
+
+suffixLenses ''Result
+
+instance Default (Result n) where
+    def = Result emptyImage [] []
+
+data RenderState n =
+    RS { viewportMap :: M.Map n Viewport
+       , rsScrollRequests :: [(n, ScrollRequest)]
+       , observedNames :: !(S.Set n)
+       , renderCache :: M.Map n (Result n)
+       }
+
 -- | The rendering context. This tells widgets how to render: how much
 -- space they have in which to render, which attribute they should use
--- to render, which bordring style should be used, and the attribute map
+-- to render, which bordering style should be used, and the attribute map
 -- available for rendering.
 data Context =
     Context { ctxAttrName :: AttrName
@@ -159,6 +200,7 @@ data Context =
             , ctxBorderStyle :: BorderStyle
             , ctxAttrMap :: AttrMap
             }
+            deriving Show
 
 suffixLenses ''RenderState
 suffixLenses ''VisibilityRequest
