@@ -1,5 +1,6 @@
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE TupleSections #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
 -- | This module provides the core widget combinators and rendering
 -- routines. Everything this library does is in terms of these basic
 -- primitives.
@@ -42,6 +43,9 @@ module Brick.Widgets.Core
   -- * Cursor placement
   , showCursor
 
+  -- * Naming
+  , Named(..)
+
   -- * Translation
   , translateBy
 
@@ -56,6 +60,7 @@ module Brick.Widgets.Core
   , visible
   , visibleRegion
   , unsafeLookupViewport
+  , cached
 
   -- ** Adding offsets to cursor positions and visibility requests
   , addResultOffset
@@ -65,16 +70,23 @@ module Brick.Widgets.Core
   )
 where
 
+#if MIN_VERSION_base(4,8,0)
+import Data.Monoid ((<>))
+#else
 import Control.Applicative
-import Control.Lens ((^.), (.~), (&), (%~), to, _1, _2, each, to, ix, Lens')
+import Data.Monoid ((<>), mempty)
+#endif
+
+import Lens.Micro ((^.), (.~), (&), (%~), to, _1, _2, each, to, ix, Lens')
+import Lens.Micro.Mtl (use, (%=))
 import Control.Monad ((>=>),when)
 import Control.Monad.Trans.State.Lazy
 import Control.Monad.Trans.Reader
 import Control.Monad.Trans.Class (lift)
 import qualified Data.Text as T
 import Data.Default
-import Data.Monoid ((<>), mempty)
 import qualified Data.Map as M
+import qualified Data.Set as S
 import qualified Data.Function as DF
 import Data.List (sortBy, partition)
 import qualified Graphics.Vty as V
@@ -87,13 +99,18 @@ import Brick.Util (clOffset, clamp)
 import Brick.AttrMap
 import Brick.Widgets.Internal
 
+-- | The class of types that store interface element names.
+class Named a n where
+    -- | Get the name of the specified value.
+    getName :: a -> n
+
 -- | When rendering the specified widget, use the specified border style
 -- for any border rendering.
-withBorderStyle :: BorderStyle -> Widget -> Widget
+withBorderStyle :: BorderStyle -> Widget n -> Widget n
 withBorderStyle bs p = Widget (hSize p) (vSize p) $ withReaderT (& ctxBorderStyleL .~ bs) (render p)
 
 -- | The empty widget.
-emptyWidget :: Widget
+emptyWidget :: Widget n
 emptyWidget = raw V.emptyImage
 
 -- | Add an offset to all cursor locations and visbility requests
@@ -105,13 +122,13 @@ emptyWidget = raw V.emptyImage
 -- by other combinators. You should call this any time you render
 -- something and then translate it or otherwise offset it from its
 -- original origin.
-addResultOffset :: Location -> Result -> Result
+addResultOffset :: Location -> Result n -> Result n
 addResultOffset off = addCursorOffset off . addVisibilityOffset off
 
-addVisibilityOffset :: Location -> Result -> Result
+addVisibilityOffset :: Location -> Result n -> Result n
 addVisibilityOffset off r = r & visibilityRequestsL.each.vrPositionL %~ (off <>)
 
-addCursorOffset :: Location -> Result -> Result
+addCursorOffset :: Location -> Result n -> Result n
 addCursorOffset off r =
     let onlyVisible = filter isVisible
         isVisible l = l^.columnL >= 0 && l^.rowL >= 0
@@ -122,7 +139,7 @@ unrestricted = 100000
 
 -- | Build a widget from a 'String'. Breaks newlines up and space-pads
 -- short lines out to the length of the longest line.
-str :: String -> Widget
+str :: String -> Widget n
 str s =
     Widget Fixed Fixed $ do
       c <- getContext
@@ -141,78 +158,94 @@ str s =
 
 -- | Build a widget from a one-line 'T.Text' value. Behaves the same as
 -- 'str'.
-txt :: T.Text -> Widget
+txt :: T.Text -> Widget n
 txt = str . T.unpack
 
 -- | Pad the specified widget on the left. If max padding is used, this
 -- grows greedily horizontally; otherwise it defers to the padded
 -- widget.
-padLeft :: Padding -> Widget -> Widget
+padLeft :: Padding -> Widget n -> Widget n
 padLeft padding p =
     let (f, sz) = case padding of
           Max -> (id, Greedy)
           Pad i -> (hLimit i, hSize p)
     in Widget sz (vSize p) $ do
-        result <- render p
+        c <- getContext
+        let lim = case padding of
+              Max -> c^.availWidthL
+              Pad i -> c^.availWidthL - i
+        result <- render $ hLimit lim p
         render $ (f $ vLimit (result^.imageL.to V.imageHeight) $ fill ' ') <+>
                  (Widget Fixed Fixed $ return result)
 
 -- | Pad the specified widget on the right. If max padding is used,
 -- this grows greedily horizontally; otherwise it defers to the padded
 -- widget.
-padRight :: Padding -> Widget -> Widget
+padRight :: Padding -> Widget n -> Widget n
 padRight padding p =
     let (f, sz) = case padding of
           Max -> (id, Greedy)
           Pad i -> (hLimit i, hSize p)
     in Widget sz (vSize p) $ do
-        result <- render p
+        c <- getContext
+        let lim = case padding of
+              Max -> c^.availWidthL
+              Pad i -> c^.availWidthL - i
+        result <- render $ hLimit lim p
         render $ (Widget Fixed Fixed $ return result) <+>
                  (f $ vLimit (result^.imageL.to V.imageHeight) $ fill ' ')
 
 -- | Pad the specified widget on the top. If max padding is used, this
 -- grows greedily vertically; otherwise it defers to the padded widget.
-padTop :: Padding -> Widget -> Widget
+padTop :: Padding -> Widget n -> Widget n
 padTop padding p =
     let (f, sz) = case padding of
           Max -> (id, Greedy)
           Pad i -> (vLimit i, vSize p)
     in Widget (hSize p) sz $ do
-        result <- render p
+        c <- getContext
+        let lim = case padding of
+              Max -> c^.availHeightL
+              Pad i -> c^.availHeightL - i
+        result <- render $ vLimit lim p
         render $ (f $ hLimit (result^.imageL.to V.imageWidth) $ fill ' ') <=>
                  (Widget Fixed Fixed $ return result)
 
 -- | Pad the specified widget on the bottom. If max padding is used,
 -- this grows greedily vertically; otherwise it defers to the padded
 -- widget.
-padBottom :: Padding -> Widget -> Widget
+padBottom :: Padding -> Widget n -> Widget n
 padBottom padding p =
     let (f, sz) = case padding of
           Max -> (id, Greedy)
           Pad i -> (vLimit i, vSize p)
     in Widget (hSize p) sz $ do
-        result <- render p
+        c <- getContext
+        let lim = case padding of
+              Max -> c^.availHeightL
+              Pad i -> c^.availHeightL - i
+        result <- render $ vLimit lim p
         render $ (Widget Fixed Fixed $ return result) <=>
                  (f $ hLimit (result^.imageL.to V.imageWidth) $ fill ' ')
 
 -- | Pad a widget on the left and right. Defers to the padded widget for
 -- growth policy.
-padLeftRight :: Int -> Widget -> Widget
+padLeftRight :: Int -> Widget n -> Widget n
 padLeftRight c w = padLeft (Pad c) $ padRight (Pad c) w
 
 -- | Pad a widget on the top and bottom. Defers to the padded widget for
 -- growth policy.
-padTopBottom :: Int -> Widget -> Widget
+padTopBottom :: Int -> Widget n -> Widget n
 padTopBottom r w = padTop (Pad r) $ padBottom (Pad r) w
 
 -- | Pad a widget on all sides. Defers to the padded widget for growth
 -- policy.
-padAll :: Int -> Widget -> Widget
+padAll :: Int -> Widget n -> Widget n
 padAll v w = padLeftRight v $ padTopBottom v w
 
 -- | Fill all available space with the specified character. Grows both
 -- horizontally and vertically.
-fill :: Char -> Widget
+fill :: Char -> Widget n
 fill ch =
     Widget Greedy Greedy $ do
       c <- getContext
@@ -222,7 +255,7 @@ fill ch =
 -- in the specified order (uppermost first). Defers growth policies to
 -- the growth policies of the contained widgets (if any are greedy, so
 -- is the box).
-vBox :: [Widget] -> Widget
+vBox :: [Widget n] -> Widget n
 vBox [] = emptyWidget
 vBox pairs = renderBox vBoxRenderer pairs
 
@@ -230,7 +263,7 @@ vBox pairs = renderBox vBoxRenderer pairs
 -- in the specified order (leftmost first). Defers growth policies to
 -- the growth policies of the contained widgets (if any are greedy, so
 -- is the box).
-hBox :: [Widget] -> Widget
+hBox :: [Widget n] -> Widget n
 hBox [] = emptyWidget
 hBox pairs = renderBox hBoxRenderer pairs
 
@@ -244,20 +277,20 @@ hBox pairs = renderBox hBoxRenderer pairs
 -- horizontal if the box layout is vertical). Doing this permits us to
 -- have one implementation for box layout and parameterizing on the
 -- orientation of all of the operations.
-data BoxRenderer =
+data BoxRenderer n =
     BoxRenderer { contextPrimary :: Lens' Context Int
                 , contextSecondary :: Lens' Context Int
                 , imagePrimary :: V.Image -> Int
                 , imageSecondary :: V.Image -> Int
-                , limitPrimary :: Int -> Widget -> Widget
-                , limitSecondary :: Int -> Widget -> Widget
-                , primaryWidgetSize :: Widget -> Size
+                , limitPrimary :: Int -> Widget n -> Widget n
+                , limitSecondary :: Int -> Widget n -> Widget n
+                , primaryWidgetSize :: Widget n -> Size
                 , concatenatePrimary :: [V.Image] -> V.Image
                 , locationFromOffset :: Int -> Location
                 , padImageSecondary :: Int -> V.Image -> V.Attr -> V.Image
                 }
 
-vBoxRenderer :: BoxRenderer
+vBoxRenderer :: BoxRenderer n
 vBoxRenderer =
     BoxRenderer { contextPrimary = availHeightL
                 , contextSecondary = availWidthL
@@ -273,7 +306,7 @@ vBoxRenderer =
                     in V.horizCat [img, p]
                 }
 
-hBoxRenderer :: BoxRenderer
+hBoxRenderer :: BoxRenderer n
 hBoxRenderer =
     BoxRenderer { contextPrimary = availWidthL
                 , contextSecondary = availHeightL
@@ -336,7 +369,7 @@ hBoxRenderer =
 -- Finally, the padded images are concatenated together vertically and
 -- returned along with the translated cursor positions and visibility
 -- requests.
-renderBox :: BoxRenderer -> [Widget] -> Widget
+renderBox :: BoxRenderer n -> [Widget n] -> Widget n
 renderBox br ws =
     Widget (maximum $ hSize <$> ws) (maximum $ vSize <$> ws) $ do
       c <- getContext
@@ -344,7 +377,17 @@ renderBox br ws =
       let pairsIndexed = zip [(0::Int)..] ws
           (his, lows) = partition (\p -> (primaryWidgetSize br $ snd p) == Fixed) pairsIndexed
 
-      renderedHis <- mapM (\(i, prim) -> (i,) <$> render prim) his
+      let availPrimary = c^.(contextPrimary br)
+          availSecondary = c^.(contextSecondary br)
+
+          renderHis _ prev [] = return prev
+          renderHis remainingPrimary prev ((i, prim):rest) = do
+              result <- render $ limitPrimary br remainingPrimary
+                               $ limitSecondary br availSecondary
+                               $ cropToContext prim
+              renderHis (remainingPrimary - (result^.imageL.(to $ imagePrimary br))) (prev ++ [(i, result)]) rest
+
+      renderedHis <- renderHis availPrimary [] his
 
       renderedLows <- case lows of
           [] -> return []
@@ -387,7 +430,7 @@ renderBox br ws =
 -- number of columns. This is important for constraining the horizontal
 -- growth of otherwise-greedy widgets. This is non-greedy horizontally
 -- and defers to the limited widget vertically.
-hLimit :: Int -> Widget -> Widget
+hLimit :: Int -> Widget n -> Widget n
 hLimit w p =
     Widget Fixed (vSize p) $
       withReaderT (& availWidthL .~ w) $ render $ cropToContext p
@@ -396,7 +439,7 @@ hLimit w p =
 -- number of rows. This is important for constraining the vertical
 -- growth of otherwise-greedy widgets. This is non-greedy vertically and
 -- defers to the limited widget horizontally.
-vLimit :: Int -> Widget -> Widget
+vLimit :: Int -> Widget n -> Widget n
 vLimit h p =
     Widget (hSize p) Fixed $
       withReaderT (& availHeightL .~ h) $ render $ cropToContext p
@@ -408,7 +451,7 @@ vLimit h p =
 -- get merged hierarchically and still fall back to the attribute map's
 -- default attribute. If you want to change the default attribute, use
 -- 'withDefAttr'.
-withAttr :: AttrName -> Widget -> Widget
+withAttr :: AttrName -> Widget n -> Widget n
 withAttr an p =
     Widget (hSize p) (vSize p) $
       withReaderT (& ctxAttrNameL .~ an) (render p)
@@ -416,7 +459,7 @@ withAttr an p =
 -- | Update the attribute map while rendering the specified widget: set
 -- its new default attribute to the one that we get by looking up the
 -- specified attribute name in the map.
-withDefAttr :: AttrName -> Widget -> Widget
+withDefAttr :: AttrName -> Widget n -> Widget n
 withDefAttr an p =
     Widget (hSize p) (vSize p) $ do
         c <- getContext
@@ -424,7 +467,7 @@ withDefAttr an p =
 
 -- | When rendering the specified widget, update the attribute map with
 -- the specified transformation.
-updateAttrMap :: (AttrMap -> AttrMap) -> Widget -> Widget
+updateAttrMap :: (AttrMap -> AttrMap) -> Widget n -> Widget n
 updateAttrMap f p =
     Widget (hSize p) (vSize p) $
         withReaderT (& ctxAttrMapL %~ f) (render p)
@@ -432,19 +475,19 @@ updateAttrMap f p =
 -- | When rendering the specified widget, force all attribute lookups
 -- in the attribute map to use the value currently assigned to the
 -- specified attribute name.
-forceAttr :: AttrName -> Widget -> Widget
+forceAttr :: AttrName -> Widget n -> Widget n
 forceAttr an p =
     Widget (hSize p) (vSize p) $ do
         c <- getContext
         withReaderT (& ctxAttrMapL .~ (forceAttrMap (attrMapLookup an (c^.ctxAttrMapL)))) (render p)
 
 -- | Build a widget directly from a raw Vty image.
-raw :: V.Image -> Widget
+raw :: V.Image -> Widget n
 raw img = Widget Fixed Fixed $ return $ def & imageL .~ img
 
 -- | Translate the specified widget by the specified offset amount.
 -- Defers to the translated width for growth policy.
-translateBy :: Location -> Widget -> Widget
+translateBy :: Location -> Widget n -> Widget n
 translateBy off p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
@@ -453,7 +496,7 @@ translateBy off p =
 
 -- | Crop the specified widget on the left by the specified number of
 -- columns. Defers to the translated width for growth policy.
-cropLeftBy :: Int -> Widget -> Widget
+cropLeftBy :: Int -> Widget n -> Widget n
 cropLeftBy cols p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
@@ -464,7 +507,7 @@ cropLeftBy cols p =
 
 -- | Crop the specified widget on the right by the specified number of
 -- columns. Defers to the translated width for growth policy.
-cropRightBy :: Int -> Widget -> Widget
+cropRightBy :: Int -> Widget n -> Widget n
 cropRightBy cols p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
@@ -474,7 +517,7 @@ cropRightBy cols p =
 
 -- | Crop the specified widget on the top by the specified number of
 -- rows. Defers to the translated width for growth policy.
-cropTopBy :: Int -> Widget -> Widget
+cropTopBy :: Int -> Widget n -> Widget n
 cropTopBy rows p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
@@ -485,7 +528,7 @@ cropTopBy rows p =
 
 -- | Crop the specified widget on the bottom by the specified number of
 -- rows. Defers to the translated width for growth policy.
-cropBottomBy :: Int -> Widget -> Widget
+cropBottomBy :: Int -> Widget n -> Widget n
 cropBottomBy rows p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
@@ -495,24 +538,49 @@ cropBottomBy rows p =
 
 -- | When rendering the specified widget, also register a cursor
 -- positioning request using the specified name and location.
-showCursor :: Name -> Location -> Widget -> Widget
+showCursor :: n -> Location -> Widget n -> Widget n
 showCursor n cloc p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
       return $ result & cursorsL %~ (CursorLocation cloc (Just n):)
 
-hRelease :: Widget -> Maybe Widget
+hRelease :: Widget n -> Maybe (Widget n)
 hRelease p =
     case hSize p of
         Fixed -> Just $ Widget Greedy (vSize p) $ withReaderT (& availWidthL .~ unrestricted) (render p)
         Greedy -> Nothing
 
-vRelease :: Widget -> Maybe Widget
+vRelease :: Widget n -> Maybe (Widget n)
 vRelease p =
     case vSize p of
         Fixed -> Just $ Widget (hSize p) Greedy $ withReaderT (& availHeightL .~ unrestricted) (render p)
         Greedy -> Nothing
 
+-- | Render the specified widget. If the widget has an entry in the
+-- rendering cache using the specified name as the cache key, use the
+-- rendered version from the cache instead. If not, render the widget
+-- and update the cache.
+--
+-- See also 'invalidateCacheEntry'.
+cached :: (Ord n) => n -> Widget n -> Widget n
+cached n w =
+    Widget (hSize w) (vSize w) $ do
+        result <- cacheLookup n
+        case result of
+            Just prevResult -> return prevResult
+            Nothing  -> do
+                wResult <- render w
+                cacheUpdate n wResult
+                return wResult
+
+cacheLookup :: (Ord n) => n -> RenderM n (Maybe (Result n))
+cacheLookup n = do
+    cache <- lift $ gets (^.renderCacheL)
+    return $ M.lookup n cache
+
+cacheUpdate :: (Ord n) => n -> Result n -> RenderM n ()
+cacheUpdate n r = lift $ modify (& renderCacheL %~ M.insert n r)
+
 -- | Render the specified widget in a named viewport with the
 -- specified type. This permits widgets to be scrolled without being
 -- scrolling-aware. To make the most use of viewports, the specified
@@ -527,15 +595,16 @@ vRelease p =
 -- taking preference. If a viewport receives more than one scrolling
 -- request from 'Brick.Main.EventM', all are honored in the order in
 -- which they are received.
-viewport :: Name
+viewport :: (Ord n, Show n)
+         => n
          -- ^ The name of the viewport (must be unique and stable for
          -- reliable behavior)
          -> ViewportType
          -- ^ The type of viewport (indicates the permitted scrolling
          -- direction)
-         -> Widget
+         -> Widget n
          -- ^ The widget to be rendered in the scrollable viewport
-         -> Widget
+         -> Widget n
 viewport vpname typ p =
     Widget Greedy Greedy $ do
       -- First, update the viewport size.
@@ -545,29 +614,42 @@ viewport vpname typ p =
           doInsert (Just vp) = Just $ vp & vpSize .~ newSize
           doInsert Nothing = Just newVp
 
+      let observeName :: (Ord n, Show n) => n -> RenderM n ()
+          observeName n = do
+              observed <- use observedNamesL
+              case S.member n observed of
+                  False -> observedNamesL %= S.insert n
+                  True -> error $ "Error: while rendering the interface, the name " <> show n <>
+                                  " was seen more than once. You should ensure that all of the widgets " <>
+                                  "in each interface have unique name values. This means either " <>
+                                  "using a different name type or adding constructors to your " <>
+                                  "existing one and using those to name your widgets.  For more " <>
+                                  "information, see the \"Widget Names\" section of the Brick User Guide."
+
+      observeName vpname
+
       lift $ modify (& viewportMapL %~ (M.alter doInsert vpname))
 
       -- Then render the sub-rendering with the rendering layout
       -- constraint released (but raise an exception if we are asked to
       -- render an infinitely-sized widget in the viewport's scrolling
       -- dimension)
-      let Name vpn = vpname
-          release = case typ of
+      let release = case typ of
             Vertical -> vRelease
             Horizontal -> hRelease
             Both ->vRelease >=> hRelease
           released = case release p of
             Just w -> w
             Nothing -> case typ of
-                Vertical -> error $ "tried to embed an infinite-height widget in vertical viewport " <> (show vpn)
-                Horizontal -> error $ "tried to embed an infinite-width widget in horizontal viewport " <> (show vpn)
-                Both -> error $ "tried to embed an infinite-width or infinite-height widget in 'Both' type viewport " <> (show vpn)
+                Vertical -> error $ "tried to embed an infinite-height widget in vertical viewport " <> (show vpname)
+                Horizontal -> error $ "tried to embed an infinite-width widget in horizontal viewport " <> (show vpname)
+                Both -> error $ "tried to embed an infinite-width or infinite-height widget in 'Both' type viewport " <> (show vpname)
 
       initialResult <- render released
 
       -- If the rendering state includes any scrolling requests for this
       -- viewport, apply those
-      reqs <- lift $ gets $ (^.scrollRequestsL)
+      reqs <- lift $ gets $ (^.rsScrollRequestsL)
       let relevantRequests = snd <$> filter (\(n, _) -> n == vpname) reqs
       when (not $ null relevantRequests) $ do
           Just vp <- lift $ gets $ (^.viewportMapL.to (M.lookup vpname))
@@ -640,7 +722,7 @@ viewport vpname typ p =
 -- is because viewports are updated during rendering and the one you are
 -- interested in may not have been rendered yet. So if you want to use
 -- this, be sure you know what you are doing.
-unsafeLookupViewport :: Name -> RenderM (Maybe Viewport)
+unsafeLookupViewport :: (Ord n) => n -> RenderM n (Maybe Viewport)
 unsafeLookupViewport name = lift $ gets (M.lookup name . (^.viewportMapL))
 
 scrollTo :: ViewportType -> ScrollRequest -> V.Image -> Viewport -> Viewport
@@ -654,6 +736,7 @@ scrollTo Vertical req img vp = vp & vpTop .~ newVStart
             VScrollPage Down -> vp^.vpTop + vp^.vpSize._2
             VScrollToBeginning -> 0
             VScrollToEnd -> V.imageHeight img - vp^.vpSize._2
+            SetTop i -> i
             _ -> vp^.vpTop
 scrollTo Horizontal req img vp = vp & vpLeft .~ newHStart
     where
@@ -664,6 +747,7 @@ scrollTo Horizontal req img vp = vp & vpLeft .~ newHStart
             HScrollPage Down -> vp^.vpLeft + vp^.vpSize._1
             HScrollToBeginning -> 0
             HScrollToEnd -> V.imageWidth img - vp^.vpSize._1
+            SetLeft i -> i
             _ -> vp^.vpLeft
 
 scrollToView :: ViewportType -> VisibilityRequest -> Viewport -> Viewport
@@ -706,7 +790,7 @@ scrollToView Horizontal rq vp = vp & vpLeft .~ newHStart
 -- deal with the details of scrolling state management.
 --
 -- This does nothing if not rendered in a viewport.
-visible :: Widget -> Widget
+visible :: Widget n -> Widget n
 visible p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
@@ -725,7 +809,7 @@ visible p =
 -- relative to the specified widget's upper-left corner of (0, 0).
 --
 -- This does nothing if not rendered in a viewport.
-visibleRegion :: Location -> V.DisplayRegion -> Widget -> Widget
+visibleRegion :: Location -> V.DisplayRegion -> Widget n -> Widget n
 visibleRegion vrloc sz p =
     Widget (hSize p) (vSize p) $ do
       result <- render p
@@ -738,19 +822,19 @@ visibleRegion vrloc sz p =
 -- | Horizontal box layout: put the specified widgets next to each other
 -- in the specified order. Defers growth policies to the growth policies
 -- of both widgets.  This operator is a binary version of 'hBox'.
-(<+>) :: Widget
+(<+>) :: Widget n
       -- ^ Left
-      -> Widget
+      -> Widget n
       -- ^ Right
-      -> Widget
+      -> Widget n
 (<+>) a b = hBox [a, b]
 
 -- | Vertical box layout: put the specified widgets one above the other
 -- in the specified order. Defers growth policies to the growth policies
 -- of both widgets.  This operator is a binary version of 'vBox'.
-(<=>) :: Widget
+(<=>) :: Widget n
       -- ^ Top
-      -> Widget
+      -> Widget n
       -- ^ Bottom
-      -> Widget
+      -> Widget n
 (<=>) a b = vBox [a, b]
